/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;

/**
 * Diese Klasse nutzt den {@link de.bsvrz.sys.funclib.bitctrl.interpreter
 * Interpreter} um mehrere Variablen gleichzeitig zu defuzzyfizieren.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class Defuzzyfizierung implements Ausdruck {

	/** Menge der zu defuzzyfizierenden Werte. */
	private final List<String> werte;

	/**
	 * Initialisierung.
	 */
	public Defuzzyfizierung() {
		werte = new ArrayList<>();
	}

	/**
	 * F&uuml;gt eine zu defuzzyfizierende Variable der Menge hinzu.
	 *
	 * @param name
	 *            Name der zu defuzzyfizierenden Fuzzy-Variable
	 */
	public void addFuzzyVariable(final String name) {
		werte.add(name);
	}

	/**
	 * Gibt immer {@code null} zur&uuml;ck, da dies ein Terminalsymbol ist.
	 * <p>
	 * {@inheritDoc}
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		return null;
	}

	/**
	 * Gibt eine Hashtabelle aus Name und Wert zur&uuml;ck. Der jeweilige Name
	 * wird von der dazugeh&ouml;rigen Fuzzy-Variable &uuml;bernommen.
	 *
	 * {@inheritDoc}
	 */
	@Override
	public Map<String, Integer> interpret(final Kontext kontext) {
		final Map<String, Integer> ergebnis = new HashMap<>();

		for (final String name : werte) {
			final FuzzyVariable fv = (FuzzyVariable) kontext.get(name,
					FuzzyVariable.class);
			ergebnis.put(name,
					fv.getLinguistischeVariable().defuzzyfiziere(fv));
		}

		return ergebnis;
	}

}
