/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib.modell;

import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzyVariable;
import de.bsvrz.sys.funclib.bitctrl.modell.AbstractDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.Datum;

/**
 * Dieses Datum kapselt einen Fuzzy-Wert und ihre Fuzzy-Sets.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class FuzzyWert extends AbstractDatum {

	/** Der Fuzzy-Wert des Datum. */
	private FuzzyVariable fuzzyWert;

	/** Der aktuelle Datenstatus. */
	private Status datenStatus = Datum.Status.UNDEFINIERT;

	@Override
	public FuzzyWert clone() {
		final FuzzyWert klon = new FuzzyWert();

		klon.setZeitstempel(getZeitstempel());
		klon.datenStatus = datenStatus;
		if (fuzzyWert != null) {
			klon.fuzzyWert = fuzzyWert.clone();
		}
		return klon;
	}

	@Override
	public Status getDatenStatus() {
		return datenStatus;
	}

	/**
	 * Gibt den fuzzyfizierten Wert zur&uuml;ck.
	 *
	 * @return der Fuzzy-Wert.
	 */
	public FuzzyVariable getFuzzyWert() {
		return fuzzyWert;
	}

	/**
	 * Legt den fuzzyfizierten Wert fest.
	 *
	 * @param fuzzyWert
	 *            der neue Fuzzy-Wert.
	 */
	public void setFuzzyWert(final FuzzyVariable fuzzyWert) {
		this.fuzzyWert = fuzzyWert;
	}

	@Override
	public String toString() {
		String s;

		s = getClass().getName() + "["; 
		s += "zeitpunkt=" + getZeitpunkt(); 
		s += ", datenStatus=" + getDatenStatus(); 
		s += " , fuzzyWert=" + fuzzyWert; 
		s += "]"; 

		return s;
	}

	/**
	 * setzt den aktuellen Datenstatus.
	 *
	 * @param datenStatus
	 *            der neue Status
	 */
	protected void setDatenStatus(final Status datenStatus) {
		this.datenStatus = datenStatus;
	}

}
