/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib.modell;

import static de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogikHandler.ODER;
import static de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogikHandler.UND;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.NumberValue;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.iav.fuzzylib.fuzzylib.inferenz.Regel;
import de.bsvrz.iav.fuzzylib.fuzzylib.inferenz.TermSymbol;
import de.bsvrz.iav.fuzzylib.fuzzylib.inferenz.Wissensbasis;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.OperationsSymbol;
import de.bsvrz.sys.funclib.bitctrl.modell.AbstractDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.AbstractParameterDatensatz;
import de.bsvrz.sys.funclib.bitctrl.modell.Datum;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UmfeldDatenMessStelle;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.StrassenTeilSegment;

/**
 * Kapselt die Parameterattributgruppe {@code atg.fuzzyRegelbasis}.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class PdFuzzyRegelbasis
		extends AbstractParameterDatensatz<PdFuzzyRegelbasis.Daten> {

	/**
	 * Kapselt die Daten des Datensatzes.
	 */
	public static class Daten extends AbstractDatum {

		/** Die linguistische Variable der Geschwindkeit. */
		private Wissensbasis wissensbasis;

		/** Der aktuelle Datenstatus. */
		private Status datenStatus = Datum.Status.UNDEFINIERT;

		@Override
		public Daten clone() {
			throw new UnsupportedOperationException();
		}

		@Override
		public Status getDatenStatus() {
			return datenStatus;
		}

		/**
		 * Gibt den Wert der Eigenschaft {@code wissensbasis} wieder.
		 *
		 * @return {@code wissensbasis}.
		 */
		public Wissensbasis getWissensbasis() {
			return wissensbasis;
		}

		/**
		 * setzt den aktuellen Datenstatus.
		 *
		 * @param datenStatus
		 *            der neue Status
		 */
		protected void setDatenStatus(final Status datenStatus) {
			this.datenStatus = datenStatus;
		}

		/**
		 * Legt den Wert der Eigenschaft {@code wissensbasis} fest.
		 *
		 * @param wissensbasis
		 *            der neue Wert.
		 */
		public void setWissensbasis(final Wissensbasis wissensbasis) {
			this.wissensbasis = wissensbasis;
		}

	}

	/** Die PID der Attributgruppe. */
	public static final String ATG_FUZZY_REGELBASIS = "atg.fuzzyRegelbasis"; 

	/** Die Attributgruppe kann von allen Instanzen gemeinsam genutzt werden. */
	private static AttributeGroup atg;

	/**
	 * Initialisiert das Objekt.
	 *
	 * @param mq
	 *            ein Messquerschnitt.
	 */
	public PdFuzzyRegelbasis(final MessQuerschnittAllgemein mq) {
		super(mq);

		if (atg == null) {
			final DataModel modell = ObjektFactory.getInstanz().getVerbindung()
					.getDataModel();
			atg = modell.getAttributeGroup(ATG_FUZZY_REGELBASIS);
			assert atg != null;
		}
	}

	/**
	 * Initialisiert das Objekt.
	 *
	 * @param sts
	 *            ein Stra&szlig;enteilsegment.
	 */
	public PdFuzzyRegelbasis(final StrassenTeilSegment sts) {
		super(sts);

		if (atg == null) {
			final DataModel modell = ObjektFactory.getInstanz().getVerbindung()
					.getDataModel();
			atg = modell.getAttributeGroup(ATG_FUZZY_REGELBASIS);
			assert atg != null;
		}
	}

	/**
	 * Initialisiert das Objekt.
	 *
	 * @param ufdms
	 *            eine Umfelddatenmessstelle.
	 */
	public PdFuzzyRegelbasis(final UmfeldDatenMessStelle ufdms) {
		super(ufdms);

		if (atg == null) {
			final DataModel modell = ObjektFactory.getInstanz().getVerbindung()
					.getDataModel();
			atg = modell.getAttributeGroup(ATG_FUZZY_REGELBASIS);
			assert atg != null;
		}
	}

	@Override
	public Daten erzeugeDatum() {
		return new Daten();
	}

	@Override
	public AttributeGroup getAttributGruppe() {
		return atg;
	}

	/**
	 * Hilfsfunktion um aus dem Datensatz mit der Regelbasis eine Pr&auml;misse
	 * zu extrahieren.
	 *
	 * @param array
	 *            Ein Feld im Datensatz, welches eine Pr&auml;misse enth&auml;lt
	 * @return Der logische Ausdruck, den die Pr&auml;misse darstellt; die
	 *         "verundeten" Variablen
	 */
	private static Ausdruck getPraemisse(final Array array) {
		// Die Liste der Prämissen der Regel durchlaufen
		final List<Ausdruck> praemisse = new ArrayList<>();

		for (int j = 0; j < array.getLength(); j++) {
			// Die Variable der Prämisse bestimmen
			String var;
			Data bedingung;
			int zyklus;

			bedingung = array.getItem(j);
			var = bedingung.getTextValue("Variable").getText(); 
			zyklus = bedingung.getUnscaledValue("Zyklus").intValue(); 
			if (zyklus > 0) {
				var += zyklus;
			}

			// Die Terme der Prämisse bestimmen
			praemisse.add(getTerme(var, bedingung.getArray("Term"))); 
		}

		if (praemisse.size() > 1) {
			// Die Variablen der Prämisse "verunden"
			return new OperationsSymbol(UND, praemisse);
		} else if (praemisse.size() == 1) {

			// Nur eine Variable vorhanden, kein "verunden"
			return praemisse.get(0);
		}

		// Keine Prämisse vorhanden
		return null;
	}

	/**
	 * Hilfsfunktion zum Extrahieren der Terme einer Variable in der
	 * Pr&auml;misse.
	 *
	 * @param variable
	 *            Name der Variable
	 * @param array
	 *            Das Feld, welches die Terme der Variable enth&auml;lt
	 * @return Die "veroderten" Terme der Variable
	 */
	private static Ausdruck getTerme(final String variable, final Array array) {
		List<TermSymbol> liste;

		liste = new ArrayList<>();
		for (int k = 0; k < array.getLength(); k++) {
			// Den Term bestimmen
			final Data name = array.getItem(k);
			final String termName = name.asTextValue().getText();

			// Neues Termsymbol gefunden
			liste.add(new TermSymbol(variable, termName));
		}

		// Die Termliste "verodern", falls notwendig
		if (liste.size() > 1) {
			return new OperationsSymbol(ODER, liste);
		}

		// Nur einem Term vorhanden, kein "verodern"
		return liste.get(0);
	}

	/**
	 * Wird nicht unterstützt!
	 *
	 * {@inheritDoc}
	 */
	@Override
	protected Data konvertiere(final Daten d) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void setDaten(final ResultData result) {
		if (!result.getDataDescription().getAttributeGroup()
				.equals(getAttributGruppe())) {
			throw new IllegalArgumentException(
					"Das Datum muss zur Attributgruppe " + ATG_FUZZY_REGELBASIS 
							+ " gehören."); 
		}

		final Daten datum = new Daten();
		if (result.hasData()) {
			final Data daten = result.getData();
			final Array regeln = daten.getArray("Regeln"); 
			final Wissensbasis wissensbasis = new Wissensbasis();

			// Die Liste der Regeln durchlaufen
			for (int i = 0; i < regeln.getLength(); i++) {
				Data d; // Hilfsvariable

				// Die Regelkonklusion bestimmen
				final Data regel = regeln.getItem(i);
				String variable;
				String term;
				d = regel.getItem("Konklusion"); 
				variable = d.getTextValue("Variable").getText(); 
				d = regel.getItem("Konklusion"); 
				term = d.getTextValue("Term").getText(); 

				// Die Regelprämisse ermitteln
				final Array array = regel.getArray("Prämisse"); 

				// Regel mit gewünschten Sicherheitsgrad erstellen
				final Float grad;
				final NumberValue sicherheitsgrad = regel
						.getScaledValue("Sicherheitsgrad");
				if (sicherheitsgrad.isNumber()) {
					grad = sicherheitsgrad.floatValue();
				} else {
					grad = null;
				}
				wissensbasis.addRegel(
						new Regel(getPraemisse(array), variable, term, grad));
			}

			datum.setWissensbasis(wissensbasis);
		}

		datum.setDatenStatus(Datum.Status.getStatus(result.getDataState()));
		datum.setZeitstempel(result.getDataTime());
		setDatum(datum);
		fireDatensatzAktualisiert(datum);
	}
}
