/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.4 Funktionen Fuzzy
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.fuzzylib.fuzzylib.modell.umfelddaten;

import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.AbstractPdFuzzySets;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.OdFuzzyWert;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UfdsWindGeschwindigkeitMittelWert;

/**
 * Kapselt die Parameterattributgruppe
 * {@code atg.ufdsParameterWindGeschwindigkeitMittelWertFuzzy}.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class PdUfdsParameterWindGeschwindigkeitMittelWertFuzzy
		extends AbstractPdFuzzySets {

	/** Der Name des Fuzzy-Werts. */
	public static final String FUZZY_WERT = "WindGeschwindigkeitMittelWert"; 

	/** Die PID der Attributgruppe. */
	public static final String ATG_UFDS_PARAMETER_WIND_GESCHWINDIGKEIT_MITTEL_WERT_FUZZY = "atg.ufdsParameterWindGeschwindigkeitMittelWertFuzzy"; 

	/** Die Attributgruppe kann von allen Instanzen gemeinsam genutzt werden. */
	private static AttributeGroup atg;

	/**
	 * Initialisiert den Datensatz.
	 *
	 * @param sensor
	 *            ein Umfelddatensensor f&uuml;r Windgeschwindigkeit.
	 */
	public PdUfdsParameterWindGeschwindigkeitMittelWertFuzzy(
			final UfdsWindGeschwindigkeitMittelWert sensor) {
		super(sensor);

		if (atg == null) {
			final DataModel modell = ObjektFactory.getInstanz().getVerbindung()
					.getDataModel();
			atg = modell.getAttributeGroup(
					ATG_UFDS_PARAMETER_WIND_GESCHWINDIGKEIT_MITTEL_WERT_FUZZY);
			assert atg != null;
		}
	}

	/**
	 * {@inheritDoc}
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.modell.Datensatz#getAttributGruppe()
	 */
	@Override
	public AttributeGroup getAttributGruppe() {
		return atg;
	}

	/**
	 * {@inheritDoc}
	 *
	 * @see de.bsvrz.iav.fuzzylib.fuzzylib.modell.PdFuzzySets#getFuzzyWert()
	 */
	@Override
	public Class<? extends OdFuzzyWert> getFuzzyWert() {
		return OdUfdsWindGeschwindigkeitMittelWertFuzzy.class;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getName() {
		return FUZZY_WERT;
	}

}
